/*
*   IOS SuperKeeper Remote Control stocks
*
*  by PM
*
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation,
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve,
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*
*/

#if defined _superkeeper_rc_included
  #endinput
#endif
#define _superkeeper_rc_included

#define SKRC_ERR_NONE 0					/* No error */
#define SKRC_ERR_PLUG 1					/* Plugin not found */
#define SKRC_ERR_VER 2					/* Invalid version */

#define SKRC_REQUIRED_VER 3

// skrc_query
// Returns SKRC_ERR_NONE if a SuperKeeper plugin was found and its version is ok, an error code otherwise
stock skrc_query()
{
	new ret = callfunc_begin("sk_rc_getversion", "superkeeper.amx");
	if (ret == -1)
		return SKRC_ERR_PLUG;
	if (ret == -2)
		return SKRC_ERR_VER;
	if (callfunc_end() != SKRC_REQUIRED_VER)
		return SKRC_ERR_VER;
	return SKRC_ERR_NONE;
}

// skrc_enable
// Enables the superkeeper plugin
stock skrc_enable()
{
	new ret = callfunc_begin("sk_rc_enable", "superkeeper.amx");
	if (ret == -1)
		return SKRC_ERR_PLUG;
	if (ret == -2)
		return SKRC_ERR_VER;
	callfunc_push_int(1);
	return callfunc_end();
}

// skrc_disable
// Disables the superkeeper plugin
stock skrc_disable()
{
	new ret = callfunc_begin("sk_rc_enable", "superkeeper.amx");
	if (ret == -1)
		return SKRC_ERR_PLUG;
	if (ret == -2)
		return SKRC_ERR_VER;
	callfunc_push_int(0);
	return callfunc_end();
}

// skrc_isenabled
// Returns 1 if superkeeper is running and enabled, 0 otherwise
stock skrc_isenabled()
{
	new ret = callfunc_begin("sk_rc_isenabled", "superkeeper.amx");
	if (ret != 1)
		return 0;
	callfunc_push_intref(ret);
	callfunc_end();
	return (ret != 0) ? 1 : 0;
}

// skrc_setff
// Sets friendly fire
stock skrc_setff(new_ff)
{
	new ret = callfunc_begin("sk_rc_setff", "superkeeper.amx");
	if (ret == -1)
		return SKRC_ERR_PLUG;
	if (ret == -2)
		return SKRC_ERR_VER;
	callfunc_push_int(new_ff);
	return callfunc_end();
}

// skrc_getff
// Gets friendly fire
// Returns 0 if friendly fire is off or an error occured, 1 if friendly fire is on
// No error should occur if skrc_query did not fail
stock skrc_getff()
{
	new ret = callfunc_begin("sk_rc_getff", "superkeeper.amx");
	if (ret != 1)
		return 0;
	callfunc_push_intref(ret);
	callfunc_end();
	return (ret != 0) ? 1 : 0;
}