/* AMX Mod X script.
*   IOS SuperKeeper
*
*  Copyright (C) 2004 Pavol "PM" Marko
*
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation,
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve,
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*
*  Version 0.3
*
*/

#include <amxmodx>
#include <amxmisc>
#include <engine>
#include <fun>				// spawn, set_user_rendering

/********** plugin info **********/
new PLINFO_NAME[] = "SuperKeeper";
new PLINFO_VERSION[] = "0.3b";
new PLINFO_AUTHORS[] = "PM";

#define PLINFO_INTVERSION 3

/********** config **********/
// Hud message channels
#define SK_HUDMSGCHAN1 123
#define SK_HUDMSGCHAN2 124
#define SK_HUDMSGCHAN3 125
// Required admin level to control SuperKeeper
#define SK_ADMINLEVEL ADMIN_LEVEL_H
// Breifing 
new SK_BRIEFING[] = "You are a super keeper!^n Bind a button to ^"+sk_fly^" to be able to fly!^n When you say /autobindfly the END button will be bound to ^"+sk_fly^"";
// Default FF
#define SK_DEFAULT_FF true

// uncomment this to remove the GFX fly effect
//#define NO_GFX_FLYEFFECT

// Sounds (without "sound/")
new FLYSOUND_PATH[] = "sk/fly.wav";
new KICKSOUND_PATH[] = "sk/kick.wav";

// Models
#if !defined NO_GFX_FLYEFFECT

new FLYEFFECTMODEL_PATH[] = "models/sk/flyeffect.mdl";
#define FLYEFFECTMODEL_ZSHIFT -14

#endif
/********** keys **********/
#define IN_SHOOT 1
#define IN_JUMP 2
#define IN_SLIDE 4
#define IN_FORWARD 8
#define IN_BACK 16
#define IN_TRAP 32
#define IN_STRAFELEFT 512
#define IN_STRAFERIGHT 1024
#define IN_PASS 2048
#define IN_SPRINT 8192
#define IN_SCORE 32768

/********** flags **********/
#define SKFL_FLYING					(1<<0)
#define SKFL_STARTED_FLYING			(1<<1)
#define SKFL_WAS_FLYING				(1<<2)
#define SKFL_SPRINT					(1<<3)

/********** others **********/
#define STAMINA_MAX 50

#define KICK_STAMINA 15
#define KICK_TIME 0.2
#define KICK_POWER 1350

// stamina / second
#define SPRINT_STAMINA 2
#define FLY_STAMINA 2


#define SKRC_NOERR 0

#define SND_STOP			(1<<5)
#define TE_PLAYERATTACHMENT			124
#define TE_KILLPLAYERATTACHMENTS	125

/********** globals **********/
new bool:g_Enabled;					// Is SuperKeeper enabled?
new bool:g_ModSupported;			// Is the mod supported?
new bool:g_FF;						// Friendly fire on?

new g_PlayerKeys[32];				// Players currently pressed keys
new g_PlayerOldKeys[32];			// From last frame
new g_PlayerKeysPressed[32];		// Pressed keys between this and last frame
new g_PlayerKeysReleased[32];		// Released keys between this and last frame

new g_SK[2];						// superkeepers
new g_SK_Stamina[2];				// Superkeeper stamina
new g_SK_Flags[2];					// Superkeeper flying?

new Float:g_GK_Spawnpos[2][3];		// goalkeeper spawn position for team1 (0) and team2 (1)
new Float:g_StaminaDist;			// distance between goal keeper spawn and ball spawn
new g_Msg_Stamina;					// Stamina (Ability message)

new Float:g_NextKick[2];			// next possible kick for both superkeepers

#if !defined NO_GFX_FLYEFFECT
new Float:g_NextFlyGfxEffect[2];	// next fly gfx effect display time.
new g_FlyEffectModel;
#endif

/********* util *********/
stock util_setdefaults()
{
	g_FF = SK_DEFAULT_FF;
	g_SK[0] = 0;
	g_SK[1] = 0;
	g_SK_Stamina[0] = STAMINA_MAX;
	g_SK_Stamina[1] = STAMINA_MAX;
	g_SK_Flags[0] = 0;
	g_SK_Flags[1] = 0;
}

stock util_zeroarray(arr[], size)
{
	new i;
	for (i = 0; i < size; ++i)
		arr[i] = 0;
}

stock util_disable()
{
	new bool:oldEnabled = g_Enabled;
	g_Enabled = false;
	util_zeroarray(g_PlayerKeys, 32);
	util_zeroarray(g_PlayerOldKeys, 32);
	util_zeroarray(g_PlayerKeysPressed, 32);
	util_zeroarray(g_PlayerKeysReleased, 32);
	g_SK[0] = 0;
	g_SK[1] = 0;
	if (oldEnabled != g_Enabled)
		print_status();
}

stock util_enable()
{
	if (g_ModSupported)
	{
		new bool:oldEnabled = g_Enabled;
		g_Enabled = true;
		if (oldEnabled != g_Enabled)
			print_status();
	}
}

stock util_concmd_print(id, text[])
{
	if (id > 0)
		client_print(id, print_console, text);
	else
		server_print(text);
}

stock util_readintarg(argnum)
{
	new tmp[32];
	read_argv(argnum, tmp, 31);
	return str_to_num(tmp);
}

stock bool:util_keydown(player, key)
{
	return (g_PlayerKeys[player] & key) != 0 ? true : false;
}

stock bool:util_keyup(player, key)
{
	return (g_PlayerKeys[player] & key) != 0 ? false : true;
}

stock bool:util_keypressed(player, key)
{
	return (g_PlayerKeysPressed[player] & key) != 0 ? false : true;
}

stock bool:util_keyreleased(player, key)
{
	return (g_PlayerKeysReleased[player] & key) != 0 ? false : true;
}

stock bool:util_movekeydown(player)
{
	return (g_PlayerKeys[player] & (IN_FORWARD | IN_BACK | IN_STRAFELEFT | IN_STRAFERIGHT | IN_JUMP | IN_SLIDE)) != 0 ? true : false;
}

stock util_stamina_bounds(i)
{
	if (g_SK_Stamina[i] > 50)
		g_SK_Stamina[i] = 50;
	if (g_SK_Stamina[i] < 0)
		g_SK_Stamina[i] = 0;
}

/********** some math **********/

// PI
#define M_PI		3.14159265358979323846
#define M_2PI		6.28318530717958647692

// angle indexes
#define	PITCH				0		// up / down
#define	YAW					1		// left / right
#define	ROLL				2		// fall over

// Thanks to John Carmack for some help ;]
stock m_anglevectors(Float:angles[3], Float:fwd[3], Float:right[3], Float:up[3])
{
	// sin (s) and cos (c) for yaw (y), pitch (p) and roll (r)
	new Float:sr, Float:sp, Float:sy, Float:cr, Float:cp, Float:cy;
	
	sy = floatsin(angles[YAW], degrees);
	cy = floatcos(angles[YAW], degrees);
	sp = floatsin(angles[PITCH], degrees);
	cp = floatcos(angles[PITCH], degrees);
	sr = floatsin(angles[ROLL], degrees);
	cr = floatcos(angles[ROLL], degrees);
	
	fwd[0] = cp*cy;
	fwd[1] = cp*sy;
	fwd[2] = -sp;
	
	right[0] = (-1*sr*sp*cy + -1*cr*-sy);
	right[1] = (-1*sr*sp*sy + -1*cr*cy);
	right[2] = -1*sr*cp;
	
	up[0] = (cr*sp*cy + -sr*-sy);
	up[1] = (cr*sp*sy + -sr*cy);
	up[2] = cr*cp;
}

/********** plugin_init and control commands **********/
public plugin_init()
{
	register_plugin(PLINFO_NAME, PLINFO_VERSION, PLINFO_AUTHORS);
	
	register_concmd("amx_sk_enable", "icmd_enable", SK_ADMINLEVEL, ": Enables SuperKeeper");
	register_concmd("amx_sk_disable", "icmd_disable", SK_ADMINLEVEL, ": Disables SuperKeeper");
	register_concmd("amx_sk_ff", "icmd_ff", SK_ADMINLEVEL, ": Turns SuperKeeper friendly fire on / off");
	
	register_clcmd("+sk_fly", "fly_on", -1, "+sk_fly");
	register_clcmd("-sk_fly", "fly_off", -1, "-sk_fly");
	
	register_clcmd("say /autobindfly", "fly_autobind", -1, ": Binds END to ^"+sk_fly^"");
	
	register_logevent("sk_died", 4, "1=committed suicide with", "2=worldspawn");
	
	set_task(1.5, "check_sk", 0, "", 0, "b");
	set_task(1.0, "inc_stamina", 0, "", 0, "b");
	
	// Check mod
	new modname[32];
	get_modname(modname, 31);
	g_ModSupported = equali(modname, "ios") ? true : false;
	
	if (!g_ModSupported)
	{
		g_Enabled = false;
		log_amx("This plugin only runs on IOS!");
		return;
	}
	else
	{
		g_Enabled = true;
	}

	g_Msg_Stamina = get_user_msgid("Ability");
	if (!g_Msg_Stamina)
	{
		g_Enabled = false;
		log_amx("Unexpected error (couldn't get msg id of ^"Ability^"). If this is a new IOS version, wait for a fix. If not, please contact the author.");
		return;
	}

	// get gk spawn positions
	new tmp = find_ent_by_class(-1, "info_team1_player1");
	if (!tmp)
	{
		g_Enabled = false;
		log_amx("Unexpected error (couldn't find info_team1_player1 in map). If this is a new IOS version, wait for a fix. If not, please contact the author.");
		return;
	}
	
	entity_get_vector(tmp, EV_VEC_origin, g_GK_Spawnpos[0]);

	tmp = find_ent_by_class(-1, "info_team2_player1");
	if (!tmp)
	{
		g_Enabled = false;
		log_amx("Unexpected error (couldn't find info_team2_player1 in map). If this is a new IOS version, wait for a fix. If not, please contact the author.");
		return;
	}
	entity_get_vector(tmp, EV_VEC_origin, g_GK_Spawnpos[1]);
	
	tmp = find_ent_by_class(-1, "info_ball_start");
	if (!tmp)
	{
		g_Enabled = false;
		log_amx("Unexpected error (couldn't find info_ball_start in map). If this is a new IOS version, wait for a fix. If not, please contact the author.");
		return;
	}
	new Float:tmpVector[3]	
	entity_get_vector(tmp, EV_VEC_origin, tmpVector);
	
	g_StaminaDist = vector_distance(tmpVector, g_GK_Spawnpos[0]);
	
	// Stamina stuff
	set_msg_block(g_Msg_Stamina, BLOCK_SET);
	register_event("Ability", "msg_stamina", "b");
		
	util_setdefaults();
}

public plugin_precache()
{
	precache_sound(FLYSOUND_PATH);
	precache_sound(KICKSOUND_PATH);
#if !defined NO_GFX_FLYEFFECT
	g_FlyEffectModel = precache_model(FLYEFFECTMODEL_PATH);
#endif
}

public icmd_enable(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_CONTINUE;

	if (!g_ModSupported)
	{
		util_concmd_print(id, " SuperKeeper only runs on IOS!");
		return PLUGIN_HANDLED;
	}

	util_enable();
	
	util_concmd_print(id, " Done.");
	
	return PLUGIN_HANDLED;
}

public icmd_disable(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_CONTINUE;
	
	if (!g_ModSupported)
	{
		util_concmd_print(id, " SuperKeeper only runs on IOS!");
		return PLUGIN_HANDLED;
	}

	util_disable();
	
	util_concmd_print(id, " Done.");
	
	return PLUGIN_HANDLED;
}

public icmd_ff(id, level, cid)
{
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_CONTINUE;
	
	if (read_argc() <= 1)
	{
		util_concmd_print(id, " Usage: amx_sk_ff <1|0>");
		return PLUGIN_HANDLED;
	}
	
	if (!g_Enabled)
	{
		util_concmd_print(id, " SuperKeeper not enabled!");
		return PLUGIN_HANDLED;
	}
	
	util_concmd_print(id, " Done.");
	
	g_FF = (util_readintarg(1) != 0) ? true : false;
	
	print_ffinfo();
	
	return PLUGIN_HANDLED;	
}

/********* remote control *********/
public sk_rc_getversion()
{
	return PLINFO_INTVERSION;
}

public sk_rc_enable(status)
{
	if (!status)
		util_disable();
	else
		util_enable();
	return SKRC_NOERR;
}

public sk_rc_isenabled(&status)
{
	status = g_Enabled ? 1 : 0;
	return SKRC_NOERR;
}

public sk_rc_setff(status)
{
	g_FF = (status != 0) ? true : false;
	print_ffinfo();
	return SKRC_NOERR;
}

public sk_rc_getff(&status)
{
	status = g_FF ? 1 : 0;
	return SKRC_NOERR;
}

/********* info to players *********/
// Print info to Keeper
stock print_skinfo(id)
{
	set_hudmessage(50, 240, 50, -1.0, 0.4, 2, 1.0, 4.0, 0.1, 1.5, SK_HUDMSGCHAN1)
	show_hudmessage(id, SK_BRIEFING);
}

// Print enabled / disabled info to all players
stock print_status()
{
	set_hudmessage(50, 240, 50, -1.0, 0.2, 2, 1.0, 4.0, 0.1, 1.5, SK_HUDMSGCHAN2)
	show_hudmessage(0, "SuperKeeper is now %s.", g_Enabled ? "enabled" : "disabled");
}

// Print ff info to all players
stock print_ffinfo()
{
	set_hudmessage(50, 240, 50, -1.0, 0.0, 2, 1.0, 4.0, 0.1, 1.5, SK_HUDMSGCHAN3)
	show_hudmessage(0, "SuperKeeper FriendlyFire is now %s.", g_FF ? "enabled" : "disabled");
}

/********* Actual stuff ********/

// make the fly gfx effect on a player if needed
#if !defined NO_GFX_FLYEFFECT
gfx_flyeffect(id, Float:curTime)
{
	if (curTime >= g_NextFlyGfxEffect[id])
	{
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY);
		write_byte(TE_PLAYERATTACHMENT);
		write_byte(g_SK[id]);
		write_coord(FLYEFFECTMODEL_ZSHIFT);
		write_short(g_FlyEffectModel);
		write_short(600);
		message_end();
		g_NextFlyGfxEffect[id] = curTime + 60.0;
	}
}
#endif

// stop the fly gfx effect
#if !defined NO_GFX_FLYEFFECT
gfx_flyeffect_stop(id)
{
	g_NextFlyGfxEffect[id] = 0.0;
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY);
	write_byte(TE_KILLPLAYERATTACHMENTS);
	write_byte(g_SK[id]);
	message_end();
	id = 0;		// to make the compiler quiet :)
}
#endif

// send superkeeper his current stamina
send_stamina(i)
{
	message_begin(MSG_ONE, g_Msg_Stamina, {0,0,0}, g_SK[i]);
	write_byte(g_SK_Stamina[i]);
	message_end();
}

public server_frame()
{
	if (!g_Enabled)
		return
	
	// keys
	new i;
	for (i=0; i < 32; ++i)
	{
		if (is_valid_ent(i))
		{
			g_PlayerKeys[i] = get_user_button(i);
			g_PlayerKeysPressed[i] = (g_PlayerKeys[i] ^ g_PlayerOldKeys[i]) & g_PlayerKeys[i];		// (new xor old) and new
			g_PlayerKeysReleased[i] = (g_PlayerKeys[i] ^ g_PlayerOldKeys[i]) & g_PlayerOldKeys[i];	// (new xor old) and old
		}
		else
		{
			g_PlayerKeys[i] = 0;
			g_PlayerOldKeys[i] = 0;
			g_PlayerKeysPressed[i] = 0;
			g_PlayerKeysReleased[i] = 0;
		}
	}
	
	new Float:curTime = halflife_time();
	new Float:vec[3];
	
	// speed & kick & fly
	for (i = 0; i < 2; ++i)
	{
		if (g_SK[i])
		{
			// make sure keeper won't die when falling
			entity_get_vector(g_SK[i], EV_VEC_origin, vec);
			if (vec[2] - g_GK_Spawnpos[0][2] < 5.0)
			{
				entity_get_vector(g_SK[i], EV_VEC_velocity, vec);
				if (vec[2] < -180.0)
					vec[2] = -50.0;
				entity_set_vector(g_SK[i], EV_VEC_velocity, vec);
			}

			if (util_keydown(g_SK[i], IN_SPRINT) && (g_SK_Stamina[i] >= SPRINT_STAMINA))
			{
				g_SK_Flags[i] |= SKFL_SPRINT;
				entity_set_float(g_SK[i], EV_FL_maxspeed, 410.0);
			}
			else
			{
				g_SK_Flags[i] &= ~SKFL_SPRINT;
				entity_set_float(g_SK[i], EV_FL_maxspeed, util_keydown(g_SK[i], IN_TRAP) ? 210.0 : 300.0);
			}
			
			if ((g_SK_Flags[i] & SKFL_STARTED_FLYING) && (g_SK_Stamina[i] >= FLY_STAMINA))
			{
				entity_get_vector(g_SK[i], EV_VEC_origin, vec)
				if (vec[2] - g_GK_Spawnpos[0][2] < 20.0)
				{
					vec[2] += 10.0;
					entity_set_vector(g_SK[i], EV_VEC_origin, vec);
				}
				g_SK_Flags[i] &= ~SKFL_STARTED_FLYING;
				
				g_SK_Stamina[i] -= SPRINT_STAMINA;
				util_stamina_bounds(i);
				
				g_SK_Flags[i] |= SKFL_WAS_FLYING;
				// start emiting sound
				emit_sound(g_SK[i], CHAN_ITEM, FLYSOUND_PATH, 1.0, ATTN_IDLE, 0, PITCH_NORM);
			}
			
			if ((g_SK_Flags[i] & SKFL_FLYING) && (g_SK_Stamina[i] >= FLY_STAMINA))
			{
				g_SK_Flags[i] |= SKFL_WAS_FLYING;
				
				vec[0] = 128.0;
				vec[1] = 128.0;
				vec[2] = 128.0;
#if !defined NO_GFX_FLYEFFECT
				gfx_flyeffect(i, curTime);
#endif
				entity_set_float(g_SK[i], EV_FL_gravity, -0.00001);
				entity_set_int(g_SK[i], EV_INT_sequence, 0);			// "deep_idle"
				if (util_movekeydown(g_SK[i]))
				{
					new Float:vec2[3];
					new Float:vec3[3];
					new Float:vec4[3];
					entity_get_vector(g_SK[i], EV_VEC_v_angle, vec);
					m_anglevectors(vec, vec2, vec3, vec4);
					vec[0] = 0.0;
					vec[1] = 0.0;
					vec[2] = 0.0;
					if (util_keydown(g_SK[i], IN_FORWARD))
					{
						vec[0] += vec2[0];
						vec[1] += vec2[1];
						vec[2] += vec2[2];
					}
					if (util_keydown(g_SK[i], IN_BACK))
					{
						vec[0] -= vec2[0];
						vec[1] -= vec2[1];
						vec[2] -= vec2[2];					
					}
					if (util_keydown(g_SK[i], IN_STRAFERIGHT))
					{
						vec[0] += vec3[0];
						vec[1] += vec3[1];
						vec[2] += vec3[2];
					}
					if (util_keydown(g_SK[i], IN_STRAFELEFT))
					{
						vec[0] -= vec3[0];
						vec[1] -= vec3[1];
						vec[2] -= vec3[2];
					}
					if (util_keydown(g_SK[i], IN_JUMP))
					{
						vec[2] += 1.0;
					}
					if (util_keydown(g_SK[i], IN_SLIDE))
					{
						vec[2] -= 1.0;
					}
					
					// normalize and multiply by speed
					// (multiply by inverted length)
					new Float:scalar = 1 / floatsqroot(vec[0]*vec[0] + vec[1]*vec[1] + vec[2]*vec[2]);
					// (and by speed)
					scalar *= util_keydown(g_SK[i], IN_SPRINT) ? 410.0 : (util_keydown(g_SK[i], IN_TRAP) ? 210.0 : 300.0);
					vec[0] *= scalar;
					vec[1] *= scalar;
					vec[2] *= scalar;

					entity_set_vector(g_SK[i], EV_VEC_velocity, vec);
				}
				else
				{	
					vec[0] = 0.0;
					vec[1] = 0.0;
					vec[2] = 0.0;
					entity_set_vector(g_SK[i], EV_VEC_velocity, vec);
					entity_set_vector(g_SK[i], EV_VEC_avelocity, vec);
				}
			}
			else
			{
				entity_set_float(g_SK[i], EV_FL_gravity, 0.0);
				if (g_SK_Flags[i] & SKFL_WAS_FLYING)
				{
					// Stop sound
					emit_sound(g_SK[i], CHAN_ITEM, FLYSOUND_PATH, 1.0, ATTN_IDLE, SND_STOP, PITCH_NORM);
					g_SK_Flags[i] &= ~SKFL_WAS_FLYING;
					// stop gfx effect
#if !defined NO_GFX_FLYEFFECT
					gfx_flyeffect_stop(i);
#endif
				}
			}
			
			if (util_keydown(g_SK[i], IN_SHOOT))
			{
				if (curTime >= g_NextKick[i] && g_SK_Stamina[i] >= KICK_STAMINA)
				{
					new Float:tl_start[3];
					new Float:tl_end[3];
					new Float:ignoredVec[3];
					// get origin of the SK
					entity_get_vector(g_SK[i], EV_VEC_origin, tl_start);
					// get his aim vector multiplied by 5
					VelocityByAim(g_SK[i], 100, tl_end);
					// add origin to it so we can use it as end point for trace_line
					tl_end[0] += tl_start[0];
					tl_end[1] += tl_start[1];
					tl_end[2] += tl_start[2];
					
					new foundPerson = trace_line(g_SK[i], tl_start, tl_end, ignoredVec);
					
					if (foundPerson > 0 && foundPerson <= 32)
					{	// found player
						if (g_FF || (entity_get_int(g_SK[i], EV_INT_team) != entity_get_int(foundPerson, EV_INT_team)))
						{
							new Float:kickVector[3];

							g_NextKick[i] = curTime + KICK_TIME;
							VelocityByAim(g_SK[i], KICK_POWER, kickVector);
							kickVector[2] += 100;
							entity_set_vector(foundPerson, EV_VEC_velocity, kickVector);
						
							g_SK_Stamina[i] -= KICK_STAMINA;
							util_stamina_bounds(i);
							send_stamina(i);
							
							// emit sound
							emit_sound(g_SK[i], CHAN_WEAPON, KICKSOUND_PATH, 1.0, ATTN_IDLE, 0, PITCH_NORM);
						}
					}
				}
			}
		}
	}
	for (i = 0; i < 32; ++i)
		g_PlayerOldKeys[i] = g_PlayerKeys[i];
}

public fly_on(id)
{
	if (!g_Enabled)
		return PLUGIN_CONTINUE;
	
	if (id == g_SK[0])
		g_SK_Flags[0] |= SKFL_FLYING | SKFL_STARTED_FLYING;
	else if (id == g_SK[1])
		g_SK_Flags[1] |= SKFL_FLYING | SKFL_STARTED_FLYING;
		
	return PLUGIN_HANDLED;
}

public fly_off(id)
{
	if (!g_Enabled)
		return PLUGIN_CONTINUE;
	
	if (id == g_SK[0])
		g_SK_Flags[0] &= ~(SKFL_FLYING | SKFL_STARTED_FLYING);
	else if (id == g_SK[1])
		g_SK_Flags[1] &= ~(SKFL_FLYING | SKFL_STARTED_FLYING);
		
	return PLUGIN_HANDLED;
}

public fly_autobind(id)
{
	if (!g_Enabled)
		return PLUGIN_CONTINUE;
		
	
	client_cmd(id, "bind ^"END^" ^"+sk_fly^"");
	client_print(id, print_chat, "* The command bind ^"END^" ^"+sk_fly^" was executed on you.");
	
	return PLUGIN_HANDLED;
}

// Check for superkeepers
public check_sk()
{
	if (!g_Enabled)
		return;
	
	new players[32];
	new playersNum;
	new modelName[64];
	new i;
	new keeper1=0, keeper2=0;
	
	get_players(players, playersNum, "");
	// get keepers
	for (i = 0; i < playersNum; ++i)
	{
		get_user_info(players[i], "model", modelName, 63);
		if (containi(modelName, "keeper") != -1)
		{
			if (keeper1)
				keeper2 = players[i];
			else
				keeper1 = players[i];
		}
	}
	if (g_SK[0] != keeper1)
	{
		g_SK[0] = keeper1;
		if (keeper1 != 0)
		{
			print_skinfo(keeper1);
		}
	}
	if (g_SK[1] != keeper2)
	{
		g_SK[1] = keeper2;
		if (keeper2 != 0)
		{
			print_skinfo(keeper2);
		}
	}
}

public inc_stamina()
{
	// increase stamina
	// decrement more
	if (!g_Enabled)
		return;
	
	new Float:pos[3];
	
	new i;
	for (i = 0; i < 2; ++i)
	{
		if (g_SK[i])
		{
			entity_get_vector(g_SK[i], EV_VEC_origin, pos);
			g_SK_Stamina[i] += floatround(g_StaminaDist*3 / vector_distance(g_GK_Spawnpos[entity_get_int(g_SK[i], EV_INT_team)], pos));
			g_SK_Stamina[i] -= floatround(g_StaminaDist*4 / vector_distance(g_GK_Spawnpos[(entity_get_int(g_SK[i], EV_INT_team)) ? 0 : 1], pos));
			if (g_SK_Flags[i] & SKFL_FLYING)
				g_SK_Stamina[i] -= FLY_STAMINA;
			if (g_SK_Flags[i] & SKFL_SPRINT)
				g_SK_Stamina[i] -= SPRINT_STAMINA;
		
			util_stamina_bounds(i);
				
			send_stamina(i);
		}
	}
}

public msg_stamina(id)
{
	if (!g_Enabled)
		return;
	// send our own msg
	message_begin(MSG_ONE, g_Msg_Stamina, {0,0,0}, id);
	write_byte((id == g_SK[0]) ? g_SK_Stamina[0] : ((id == g_SK[1]) ? g_SK_Stamina[1] : read_data(1)));
	message_end();
}

public plugin_pause()
{
	// unblock stamina
	set_msg_block(g_Msg_Stamina, BLOCK_NOT);
}

public plugin_unpause()
{
	// block stamina
	set_msg_block(g_Msg_Stamina, BLOCK_SET);
}


// the superkeeper died after falling
public sk_died()
{
	new user[128];
	new userId;
	read_logargv(0, user, 127);
	parse_loguser(user, "", 0, userId);
	
	new params[2];
	params[0] = userId;
	params[1] = 1;
	set_task(0.5, "respawnsk", 0, params, 2);
}

public respawnsk(params[2])
{
	spawn(params[0]);
	if (params[1])
	{
		params[1] = 0;
		set_task(0.1, "respawnsk", 0, params, 2);
	}
}
