/* AMX Mod X script.
*   IOS SuperKeeper Vote
*
*  Copyright (C) 2004 Pavol "PM" Marko
*
*  This program is free software; you can redistribute it and/or modify it
*  under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 2 of the License, or (at
*  your option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
*  General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software Foundation,
*  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*  In addition, as a special exception, the author gives permission to
*  link the code of this program with the Half-Life Game Engine ("HL
*  Engine") and Modified Game Libraries ("MODs") developed by Valve,
*  L.L.C ("Valve"). You must obey the GNU General Public License in all
*  respects for all of the code used other than the HL Engine and MODs
*  from Valve. If you modify this file, you may extend this exception
*  to your version of the file, but you are not obligated to do so. If
*  you do not wish to do so, delete this exception statement from your
*  version.
*
*  Version 0.3
*
*/

#include <amxmodx>
#include <amxmisc>
#include <superkeeper_rc>

new PLINFO_NAME[] = "SuperKeeper Vote"
new PLINFO_VERSION[] = "0.3"
new PLINFO_AUTHORS[] = "PM"
#define PLINFO_INTVERSION 3

new g_Option[2];
new g_FFOption[2];
new bool:g_SK_IsOK;

public plugin_init()
{
	register_plugin(PLINFO_NAME, PLINFO_VERSION, PLINFO_AUTHORS);
	// We can only choose yes and no, so the keys are (1<<0) | (1<<1), that would be 3 :)
	register_menucmd(register_menuid("Enable SuperKeeper?"), (1<<0) | (1<<1), "skvote_voted");
	register_clcmd("say /votesk", "skvote_start", 0, "Start a SuperKeeper vote");
	register_clcmd("say /votesuperkeeper", "skvote_start", 0, "Start a SuperKeeper vote");
	register_clcmd("say /skvote", "skvote_start", 0, "Start a SuperKeeper vote");
	register_clcmd("say /superkeepervote", "skvote_start", 0, "Start a SuperKeeper vote");
	register_clcmd("skvote", "skvote_start", 0, "Start a SuperKeeper vote");
	register_clcmd("votesk", "skvote_start", 0, "Start a SuperKeeper vote");
	
	register_menucmd(register_menuid("Enable SuperKeeper FF?"), (1<<0) | (1<<1), "skffvote_voted");
	register_clcmd("say /voteskff", "skffvote_start" ,0, "Start a SuperKeeper FF vote");
	register_clcmd("say /skffvote", "skffvote_start" ,0, "Start a SuperKeeper FF vote");
	register_clcmd("voteskff", "skffvote_start" ,0, "Start a SuperKeeper FF vote");
	register_clcmd("skffvote", "skffvote_start" ,0, "Start a SuperKeeper FF vote");
	
	register_cvar("amx_sk_votes", "1.0");
	
	switch (skrc_query())
	{
		case SKRC_ERR_PLUG:
		{
			log_amx("superkeeper.amx not running!");
			g_SK_IsOK = false;
		}
		case SKRC_ERR_VER:
		{
			log_amx("The version of your superkeeper.amx is incompatible with this plugin.");
			g_SK_IsOK = false;
		}
		default:
			g_SK_IsOK = true;
	}
}

public skvote_start(id)
{
	if (!g_SK_IsOK)
	{
		client_print(id, print_chat, "* The SuperKeeper Vote plugin is not able to control SuperKeeper.");
		return PLUGIN_HANDLED;
	}
	
	if (!get_cvar_num("amx_sk_votes"))
	{
		client_print(id, print_chat, "* SuperKeeper Votes are disabled.");
		return PLUGIN_HANDLED;
	}
	
	// Make sure there is amx_vote_delay time between two votes and that no vote is running at the moment.
	new Float:lastVoting = get_cvar_float("amx_last_voting");
	if (lastVoting > get_gametime())
	{
		client_print(id, print_chat, "* There is already one voting...");
		return PLUGIN_HANDLED;
	}
	if (lastVoting && (lastVoting+get_cvar_float("amx_vote_delay") > get_gametime()))
	{
		client_print(id, print_chat, "* Voting not allowed at this time...");
		return PLUGIN_HANDLED;
	}
	
	// Clear options (so we don't get garbage values :) )
	g_Option[0] = 0;
	g_Option[1] = 0;
	
	// Make sure no vote gets started before amx_vote_time elapsed
	new Float:voteTime = get_cvar_float("amx_vote_time") + 2.0;
	set_cvar_float("amx_last_voting", get_gametime() + voteTime);
	
	// Show the menu
	show_menu(0, (1<<0) | (1<<1), "[AMX]: Enable SuperKeeper?^n^n1. Yes^n2. No", floatround(voteTime));
	// Make a task that will count the votes when voting is finished
	set_task(voteTime, "skvote_done");
	
	// Display who started the vote
	new name[32];
	get_user_name(id, name, 31);
	client_print(0, print_chat, "* %s started a SuperKeeper vote...", name)
	
	return PLUGIN_HANDLED;
}

public skvote_voted(id, key)
{
	// Show who voted for what if needed
	if (get_cvar_float("amx_vote_answers"))
	{
		new name[32];
		get_user_name(id, name, 31);
		client_print(0, print_chat, "* %s voted to %s", name, key ? "disable SuperKeeper" : "enable SuperKeeper");
	}
	// Increment option
	++g_Option[key];
	
	return PLUGIN_HANDLED;
}

public skvote_done()
{
	if (g_Option[0] > g_Option[1])
	{
		client_print(0, print_chat, "* Voting successful (yes ^"%d^") (no ^"%d^").", g_Option[0], g_Option[1]);
		skrc_enable();
	}
	else
	{
		client_print(0, print_chat, "* Voting failed (yes ^"%d^") (no ^"%d^").", g_Option[0], g_Option[1]);
		skrc_disable();
	}
}


public skffvote_start(id)
{
	if (!g_SK_IsOK)
	{
		client_print(id, print_chat, "* The SuperKeeper Vote plugin is not able to control SuperKeeper.");
		return PLUGIN_HANDLED;
	}
	
	if (!get_cvar_num("amx_sk_votes"))
	{
		client_print(id, print_chat, "* SuperKeeper Votes are disabled.");
		return PLUGIN_HANDLED;
	}
	
	// Make sure there is amx_vote_delay time between two votes and that no vote is running at the moment.
	new Float:lastVoting = get_cvar_float("amx_last_voting");
	if (lastVoting > get_gametime())
	{
		client_print(id, print_chat, "* There is already one voting...");
		return PLUGIN_HANDLED;
	}
	if (lastVoting && (lastVoting+get_cvar_float("amx_vote_delay") > get_gametime()))
	{
		client_print(id, print_chat, "* Voting not allowed at this time...");
		return PLUGIN_HANDLED;
	}
	
	// Clear options (so we don't get garbage values :) )
	g_FFOption[0] = 0;
	g_FFOption[1] = 0;
	
	// Make sure no vote gets started before amx_vote_time elapsed
	new Float:voteTime = get_cvar_float("amx_vote_time") + 2.0;
	set_cvar_float("amx_last_voting", get_gametime() + voteTime);
	
	// Show the menu
	show_menu(0, (1<<0) | (1<<1), "[AMX]: Enable SuperKeeper FF?^n^n1. Yes^n2. No", floatround(voteTime));
	// Make a task that will count the votes when voting is finished
	set_task(voteTime, "skffvote_done");
	
	// Display who started the vote
	new name[32];
	get_user_name(id, name, 31);
	client_print(0, print_chat, "* %s started a SuperKeeper vote...", name);
	
	return PLUGIN_HANDLED;
}

public skffvote_voted(id, key)
{
	// Show who voted for what if needed
	if (get_cvar_float("amx_vote_answers"))
	{
		new name[32];
		get_user_name(id, name, 31);
		client_print(0, print_chat, "* %s voted to %s", name, key ? "disable SuperKeeper FF" : "enable SuperKeeper FF");
	}
	// Increment option
	++g_FFOption[key];
	
	return PLUGIN_HANDLED;
}

public skffvote_done()
{
	if (g_FFOption[0] > g_FFOption[1])
	{
		client_print(0, print_chat, "* Voting successful (yes ^"%d^") (no ^"%d^").", g_FFOption[0], g_FFOption[1]);
		skrc_setff(1);
	}
	else
	{
		client_print(0, print_chat, "* Voting failed (yes ^"%d^") (no ^"%d^").", g_FFOption[0], g_FFOption[1]);
		skrc_setff(0);
	}
}